"""
Eventlog device (not really a device)
"""

# this import is depending on python version
try:
    import HTMLParser
    HTMLPARSER = HTMLParser.HTMLParser
except ImportError:
    import html.parser
    HTMLPARSER = html.parser.HTMLParser

EVENTLOG_URL = '/uk/eventlog.html'
EVENTLOG_ITEMS_URL = '/eventlog/eventlog_items.dyn'
EVENTLOG_FILTER_URL = '/eventlog/filterEventLog.cmd'


class Eventlog(object):
    """ Eventlog device

    Args:
        session (verisure.session): Current session
    """

    def __init__(self, session):
        self._session = session

    def get(self, pages=1, offset=0, *logfilter):
        """ Get event log

        Args:
            pages (int): Number of pages to request
            offset (int): Page offset
            *logfilter (str): [ARM,DISARM,FIRE,INTRUSION,TECHNICAL,SOS,WARNING]
        """
        self._session.get(EVENTLOG_URL, to_json=False)
        if logfilter:
            self._session.post(
                EVENTLOG_FILTER_URL,
                data={'eventLogFilter': logfilter})
        parser = EventlogParser()
        for page in range(1, pages + 1):
            parser.feed(self._session.get(
                EVENTLOG_ITEMS_URL,
                to_json=False,
                eventlogPage=page,
                offset=offset))
        return parser.eventlog


class EventlogParser(HTMLPARSER):
    """ Parse relevant information from HTML response """
    def __init__(self):
        HTMLPARSER.__init__(self)
        self.eventlog = []
        self.eventlogitem = {}
        self.recording = False

    def handle_starttag(self, tag, attributes):
        if tag != 'div':
            return
        for name, value in attributes:
            self.recording = name == 'class' and (
                value == 'eventlog-list--title' or
                value == 'eventlog-list--datetime' or
                value == 'eventlog-list--details-text hidden')

    def handle_endtag(self, tag):
        if tag == 'div':
            self.recording = False

    def handle_data(self, data):
        if not self.recording or not data.strip():
            return
        if len(self.eventlogitem) == 0:
            self.eventlogitem['title'] = data.strip()
        elif len(self.eventlogitem) == 1:
            self.eventlogitem['date'] = data.strip()
        elif len(self.eventlogitem) == 2:
            self.eventlogitem['device'] = data
        elif len(self.eventlogitem) == 3:
            self.eventlogitem['location'] = data.replace('-', '').strip()
        elif len(self.eventlogitem) == 4:
            self.eventlogitem['details'] = data.strip()
            self.eventlog.append(self.eventlogitem)
            self.eventlogitem = {}



""" Command line interface for Verisure MyPages """

from __future__ import print_function

import argparse
import pprint

from verisure import MyPages

COMMAND_GET = 'get'
COMMAND_SET = 'set'
COMMAND_HISTORY = 'history'
COMMAND_EVENTLOG = 'eventlog'


def print_overviews(overviews):
    ''' print the overviews of devices '''
    if isinstance(overviews, list):
        for overview in overviews:
            print_overview(overview)
    else:
        print_overview(overviews)


def print_overview(overview):
    """ print the overview of a device """
    print(overview.get_typename())
    for key, value in overview.get_status():
        print(u'\t{}: {}'.format(key, value))


def main():
    """ Start verisure command line """
    parser = argparse.ArgumentParser(
        description='Read or change status of verisure devices')
    parser.add_argument(
        'username',
        help='MySite username')
    parser.add_argument(
        'password',
        help='MySite password')

    commandsparser = parser.add_subparsers(
        help='commands',
        dest='command')

    # Get command
    get_parser = commandsparser.add_parser(
        COMMAND_GET,
        help='Read status of one or many device types')
    get_parser.add_argument(
        'devices',
        nargs='*',
        choices=[
            'alarm',
            'climate',
            'ethernet',
            'lock',
            'mousedetection',
            'nest',
            'smartcam',
            'smartplug',
            'temperaturecontrol',
            'vacationmode',
            'all'
        ],
        help='Read status for device type')

    # Set command
    set_parser = commandsparser.add_parser(
        COMMAND_SET,
        help='Set status of a device')
    set_device = set_parser.add_subparsers(
        help='device',
        dest='device')

    # Set smartplug
    set_smartplug = set_device.add_parser(
        'smartplug',
        help='set smartplug value')
    set_smartplug.add_argument(
        'serial_number',
        help='serial number')
    set_smartplug.add_argument(
        'new_value',
        choices=[
            'on',
            'off'],
        help='new value')

    # Set alarm
    set_alarm = set_device.add_parser(
        'alarm',
        help='set alarm status')
    set_alarm.add_argument(
        'code',
        help='alarm code')
    set_alarm.add_argument(
        'new_status',
        choices=[
            'ARMED_HOME',
            'ARMED_AWAY',
            'DISARMED'],
        help='new status')

    # Set lock
    set_lock = set_device.add_parser(
        'lock',
        help='set lock status')
    set_lock.add_argument(
        'code',
        help='alarm code')
    set_lock.add_argument(
        'serial_number',
        help='serial number')
    set_lock.add_argument(
        'new_status',
        choices=[
            'LOCKED',
            'UNLOCKED'],
        help='new status')

    # History command
    history_parser = commandsparser.add_parser(
        COMMAND_HISTORY,
        help='Get history of a device')
    history_device = history_parser.add_subparsers(
        help='device',
        dest='device')

    # Get climate history
    history_climate = history_device.add_parser(
        'climate',
        help='get climate history')
    history_climate.add_argument(
        'serial_numbers',
        nargs='+',
        help='serial numbers')

    # Event log command
    eventlog_parser = commandsparser.add_parser(
        COMMAND_EVENTLOG,
        help='Get event log')
    eventlog_parser.add_argument(
        '-p', '--pages',
        type=int,
        default=1,
        help='Number of pages to request')
    eventlog_parser.add_argument(
        '-o', '--offset',
        type=int,
        default=0,
        help='Page offset')
    eventlog_parser.add_argument(
        '-f', '--filter',
        nargs='*',
        default=[],
        choices=[
            'ARM',
            'DISARM',
            'FIRE',
            'INTRUSION',
            'TECHNICAL',
            'SOS',
            'WARNING'],
        help='Filter event log')

    args = parser.parse_args()

    with MyPages(args.username, args.password) as verisure:
        if args.command == COMMAND_GET:
            if 'all' in args.devices:
                print_overviews(verisure.get_overviews())
            else:
                for dev in args.devices:
                    print_overviews(verisure.__dict__[dev].get())
        if args.command == COMMAND_SET:
            if args.device == 'smartplug':
                print(verisure.smartplug.set(
                    args.serial_number,
                    args.new_value))
            if args.device == 'alarm':
                print(verisure.alarm.set(
                    args.code,
                    args.new_status))
            if args.device == 'lock':
                print(verisure.lock.set(
                    args.code,
                    args.serial_number,
                    args.new_status))
        if args.command == COMMAND_HISTORY:
            if args.device == 'climate':
                pprint.PrettyPrinter().pprint(verisure.climate.get_history(
                    *args.serial_numbers))
        if args.command == COMMAND_EVENTLOG:
            pprint.PrettyPrinter().pprint(
                verisure.eventlog.get(args.pages, args.offset, *args.filter))


# pylint: disable=C0103
if __name__ == "__main__":
    main()



""" submodule containing supported devices """

__all__ = [
    'Alarm',
    'Climate',
    'Ethernet',
    'Eventlog',
    'Lock',
    'Mousedetection',
    'Nest',
    'Smartcam',
    'Smartplug',
    'Temperaturecontrol',
    'Vacationmode',
]

from .alarm import Alarm
from .climate import Climate
from .ethernet import Ethernet
from .eventlog import Eventlog
from .lock import Lock
from .mousedetection import Mousedetection
from .nest import Nest
from .smartcam import Smartcam
from .smartplug import Smartplug
from .temperaturecontrol import Temperaturecontrol
from .vacationmode import Vacationmode



"""
Nest device
"""

from .overview import Overview

OVERVIEW_URL = '/overview/nest'


class Nest(object):
    """ Nest device

    Args:
        session (verisure.session): Current session
    """
    def __init__(self, session):
        self._session = session

    def get(self):
        """ get device overview """
        status = self._session.get(OVERVIEW_URL)
        return [Overview('nest', status)]
